/*******************************************************************
 * Xatm -- OCF, CID, TrueType scalable font support for X 
 *  By:  ͵ <y5t0964@students.chiba-u.ac.jp>
 */

/* This code is derived from X-VFlib's vffuncs.c */

/*$Id: Type1_16funcs.c,v 1.23 1998/05/31 15:03:58 elca Exp $*/

#include "Type1_16.h"

/* X εư -deferglyphs all ץѤ뤳 */
#undef PreloadAllGlyphs

/*#define dprintf(str) fprintf str*/
#define dprintf(str) /* */

/*#define DEBUG*/
#undef DEBUG

/*#define DUMP*/
#undef DUMP

#ifdef DUMP
#include "dump.c"
#endif /* DUMP */

/**********************************************************************
 * ؿ
 */
static void convertNothing(Type1_16FontPtr ptr, char *p);
static void convertBitOrder(Type1_16FontPtr ptr, char *p);
static void convertByteOrder(Type1_16FontPtr ptr, char *p);
static void convertBitByteOrder(Type1_16FontPtr ptr, char *p);
static void Type1_16GetBitmap(FontPtr font,
			      unsigned int c,
			      unsigned int hbits,
			      unsigned int wbytes,
			      char *bits);
static int Type1_16MakeProps (FontInfoPtr ptr, FontScalablePtr vals);
static int Type1_16Font_Init(FontPtr font,
			     char* dir,
			     char* file,
			     double matrix[4]);
static void Type1_16Font_Deinit(FontPtr font);
static int Type1_16SetupRowCol(FontPtr         font,
			       FontEntryPtr    entry,
			       FontScalablePtr vals);
static int t1_16_get_glyphs (FontPtr font,
			     unsigned long count,
			     unsigned char *chars,
			     FontEncoding encoding,
			     unsigned long *pCount,
			     CharInfoPtr *glyphs);
static int t1_16_get_metrics (FontPtr font,
			      unsigned long count,
			      unsigned char *chars,
			      FontEncoding encoding,
			      unsigned long *pCount,
			      xCharInfo **glyphs);
static void t1_16_unload_font (FontPtr font);
static void t1_16_unload_glyphs (FontPtr font);

/***************************************************************************/

/*
 * bit/byte order convert functions
 */

static void convertNothing(Type1_16FontPtr ptr, char *p)
{
}

static void convertBitOrder(Type1_16FontPtr ptr, char *p)
{
	BitOrderInvert(p, ptr->wbytes * ptr->hbits);
}

static void convertByteOrder(Type1_16FontPtr ptr, char *p)
{
	if (ptr->font->bit != ptr->font->byte) {
		switch (ptr->font->scan) {
		case 1:
			break;
		case 2:
			TwoByteSwap(p, ptr->wbytes * ptr->hbits);
		case 4:
			FourByteSwap(p, ptr->wbytes * ptr->hbits);
		}
	}
}

static void convertBitByteOrder(Type1_16FontPtr ptr, char *p)
{
	convertBitOrder(ptr, p);
	convertByteOrder(ptr, p);
}

/*--------------------------------------------------------------*/
/* ghostscript ӥåȥޥåפ
 */
static void Type1_16GetBitmap(FontPtr font,
			      unsigned int c,
			      unsigned int hbits,
			      unsigned int wbytes,
			      char *bits)
{
	Type1_16FontPtr ptr = (Type1_16FontPtr) font->fontPrivate;
	char gscmd[512];
	char *tmp_bits = bits;
	int len, totallen=0;
	double x_delta, y_delta;
	int i;
	GsPipe gspipe;

	/* ޡ */
	alarm(0);

	gspipe = gsGetPipe();

	/* Ѵˤӥåȥޥåפ̳˽ФƤޤ
	   򤫤Ƹ褦ˤ */
	x_delta = computeXdelta(ptr->matrix);
	y_delta = computeYdelta(ptr->matrix)*0.75; /* 0.75 = Ĵο */


	/* ghostscript ΥޥɤȤΩƤ */
	/* "16 16 .setpagesize /Wadalab-mincho-0-8 findfont
	   [16 0 16 16 0 0] makefont setfont 0 1 moveto
	   <3021> show showmage\n" -> "" */
	sprintf(gscmd, "%d %d .setpagesize /%s findfont [%g %g %g %g %g %g] makefont setfont 0 %g moveto <%04x> show showpage\n",
		wbytes*8, hbits,  ptr->fontname,
		ptr->matrix[0], ptr->matrix[1], ptr->matrix[2], ptr->matrix[3],
		x_delta, y_delta,
		hbits*0.156, c); /* 0.156 = Ĵο */

	dprintf((stdout, __FUNCTION__ ":gscmd %s\n", gscmd));
	dprintf((stdout, "hbits %d, wbytes %d\n", hbits, wbytes));

	/* gs->xfs Υѥץ饤Хåե򥯥ꥢ */
	_flush(gspipe.r);

	write(gspipe.w, gscmd, strlen(gscmd));

	/*usleep(500);*/

	/* XXX: ʸˤĤ 1ms * 10000 = 10 ǥॢ (X ǤޤΤɤ) */
	for (i = 0; i < 10000; i++) {
		len = read(gspipe.r, tmp_bits, hbits*wbytes);
		if (len > 0) {
			totallen += len;
			tmp_bits += len;
		}
		if(totallen >= hbits * wbytes) {
			break;
		}

		usleep(1);
	}
	dprintf((stdout, "total %d\n", totallen));

	/* ޡ */
	alarm(GS_WAIT_SEC);
}



/************************************************************************
 * Call back functions
 */

static int t1_16_get_glyphs (FontPtr font,
			     unsigned long count,
			     unsigned char *chars,
			     FontEncoding encoding,
			     unsigned long *pCount,
			     CharInfoPtr *glyphs)
{

	Type1_16FontPtr ptr = (Type1_16FontPtr) font->fontPrivate;
	CharInfoPtr charInfo;
	CharInfoPtr *glyphsBase = glyphs;
	unsigned int c;
	int hbits = ptr->hbits;
	int wbytes = ptr->wbytes;

	dprintf((stdout, "+t1_16_get_glyphs"));

	switch (encoding) {
	case Linear8Bit:
	case TwoD8Bit:
		break;
	case Linear16Bit:
	case TwoD16Bit:
		while (count--) {
			c = *chars++ << 8;
			c |= *chars++;

			dprintf((stdout, "0x%x ", c));

			if (!BTree_Search(ptr->btree, c, (void **)&charInfo)) {
				charInfo = CharInfoPool_Get(ptr->pool);
				charInfo->metrics = font->info.maxbounds;
				Type1_16GetBitmap(font, c, hbits, wbytes, charInfo->bits);
				(*ptr->convert)(ptr, charInfo->bits);
				BTree_Insert(ptr->btree, c, charInfo);
				/*BTree_Print(ptr->btree);*/

			}
			*glyphs++ = charInfo;
		}
		break;
	}
	*pCount = glyphs - glyphsBase;

	dprintf((stdout, "pCount: %d\n", *pCount));
	dprintf((stdout, "-t1_16_get_glyphs()\n\n"));

	return Successful;

}

static int t1_16_get_metrics (FontPtr font,
			      unsigned long count,
			      unsigned char *chars,
			      FontEncoding encoding,
			      unsigned long *pCount,
			      xCharInfo **glyphs)
{
	xCharInfo **glyphsBase = glyphs;
#ifdef PreloadAllGlyphs
	Type1_16FontPtr ptr = (Type1_16FontPtr) font->fontPrivate;
	CharInfoPtr charInfo;
	unsigned int c;
	int hbits = ptr->hbits;
	int wbytes = ptr->wbytes;
#endif

	dprintf((stdout, "\n+t1_16_get_metrics(0x%x, %d, 0x%x, %d, 0x%x, 0x%x)\n",
		 font, count, chars, encoding ,count, glyphs));

	switch (encoding) {
	case Linear8Bit:
	case TwoD8Bit:
		break;
	case Linear16Bit:
	case TwoD16Bit:
		while (count--) {
#ifdef PreloadAllGlyphs
			c = *chars++ << 8;
			c |= *chars++;
			if (!BTree_Search(ptr->btree, c, (void **)&charInfo)) {
				charInfo = CharInfoPool_Get(ptr->pool);
				charInfo->metrics = font->info.maxbounds;
				Type1_16GetBitmap(font, c, hbits, wbytes, charInfo->bits);
				(*ptr->convert)(ptr, charInfo->bits);
				BTree_Insert(ptr->btree, c, charInfo);
			}
			*glyphs++ = &charInfo->metrics;
#else
                        /*
			  c = *chars++ << 8; c |= *chars++;
			  dprintf((stderr, "0x%x ", c));
			  */
			*glyphs++ = &font->info.maxbounds;
#endif /* PreloadAllGlyphs */
		}
		break;
	}
	*pCount = glyphs - glyphsBase;

	dprintf((stdout, "pCount: %d\n", *pCount));
	dprintf((stdout, "-t1_16_get_metrics()\n\n"));

	return Successful;
}

static void t1_16_unload_font (FontPtr font)
{
	dprintf((stdout, "\n+t1_16_unload_font(%x)\n", font));
	Type1_16Font_Deinit(font);
	dprintf((stdout, "-t1_16_unload_font()\n\n"));
}

static void t1_16_unload_glyphs (FontPtr font)
{
	dprintf((stdout, "\n+t1_16_unload_glyphs(%x)\n", font));
	dprintf((stdout, "-t1_16_unload_glyphs()\n\n"));
}

/***************************************************************************/
/* ץѥƥϢ
 */
typedef struct {char *name; int flag; } PropTable;
#define A_NAME            0
#define A_NAME_LAST       1
#define A_NUMBER          2
#define A_NUMBER_PIXEL    3
#define A_NUMBER_POINT    4

static PropTable propTable[] = {
	{"FOUNDRY",          A_NAME},
	{"FAMILY_NAME",      A_NAME},
	{"WEIGHT_NAME",      A_NAME},
	{"SLANT",            A_NAME},
	{"SETWIDTH_NAME",    A_NAME},
	{"ADD_STYLE_NAME",   A_NAME},
	{"PIXEL_SIZE",       A_NUMBER_PIXEL},
	{"POINT_SIZE",       A_NUMBER_POINT},
	{"RESOLUTION_X",     A_NUMBER},
	{"RESOLUTION_Y",     A_NUMBER},
	{"SPACING",          A_NAME},
	{"AVERAGE_WIDTH",    A_NUMBER},
	{"CHARSET_REGISTRY", A_NAME},
	{"CHARSET_ENCODING", A_NAME_LAST},
};

/* propTable + "FONT" + "COPYRIGHT" + "X_HEIGHT" + "QUAD_WIDTH" = 18 */
#define NPROPS (XtNumber(propTable) + 4)

static int Type1_16MakeProps (FontInfoPtr ptr, FontScalablePtr vals)
{
	FontPropPtr props;
	int i, n = 0;
	char *is_str;
	char *cur, *next;

	dprintf((stdout, "\n+" __FUNCTION__ "()\n" ));

	/* props */
	ptr->props = (FontPropPtr) xalloc(sizeof(FontPropRec) * NPROPS);
	if (ptr->props == NULL) {
		return -1;
	}
	memset(ptr->props, 0, sizeof(FontPropRec) * NPROPS);
	props = ptr->props;

	/* isStringProp */
	ptr->isStringProp = (char *) xalloc(sizeof(char) * NPROPS);
	if (ptr->isStringProp == NULL) {
		xfree(ptr->props);
		return -1;
	}
	memset(ptr->isStringProp, 0, sizeof(char) * NPROPS);
	is_str = ptr->isStringProp;

	props[n].name = MakeAtom("FONT", sizeof("FONT") - 1, TRUE);
	props[n].value = MakeAtom(vals->xlfdName, strlen(vals->xlfdName), TRUE);
	*is_str = TRUE;

	is_str++;
	n++;

	cur = strchr(vals->xlfdName, '-') + 1;

	for (i = 0; i < XtNumber(propTable); i++) {

		dprintf((stdout, "."));


		next = strchr(cur, '-');
		if(next == NULL && propTable[i].flag != A_NAME_LAST) {
			break;
		}

		props[n].name = MakeAtom(propTable[i].name, strlen(propTable[i].name), TRUE);

		switch(propTable[i].flag) {
		case A_NAME: /* ʸ */
			*is_str = TRUE;
			if(next >= cur && *cur != '*') {
				props[n].value = MakeAtom(cur, next - cur, TRUE);
				dprintf((stdout, "Atom %s %s\n", propTable[i].name, cur));
			} else {
				props[n].value = MakeAtom(NULL, 0, TRUE);
			}
			break;
		case A_NAME_LAST: /* ʸ */
			*is_str = TRUE;
			if(*cur != '*') {
				props[n].value = MakeAtom(cur, strlen(cur), TRUE);
				dprintf((stdout, "Atom %s %s\n", propTable[i].name, cur));
			} else {
				props[n].value = MakeAtom(NULL, 0, TRUE);
			}
			break;
		case A_NUMBER: /*  */
			if(next >= cur && *cur != '*') {
				props[n].value = atoi(cur);
				dprintf((stdout, "num %d\n", atoi(cur)));
			} else {
				props[n].value = 0;
			}
			break;
		case A_NUMBER_PIXEL:
			props[n].value = d_to_i(computeChHeight(vals->pixel_matrix));
			break;
		case A_NUMBER_POINT:
			props[n].value = d_to_i(computeChHeight(vals->point_matrix) * 10.0);
			break;
		default:
			printf("prop unknown\n");
			break;
		}

		cur = next + 1;
		is_str++;
		n++;
	}

	props[n].name = MakeAtom("COPYRIGHT", sizeof("COPYRIGHT") - 1, TRUE);
	props[n].value = MakeAtom("Unknown: See Manual", sizeof("Unknown: See Manual") - 1, TRUE);
	*is_str = TRUE;

	is_str++;
	n++;

	props[n].name = MakeAtom("X_HEIGHT", sizeof("X_HEIGHT") - 1, TRUE);
	props[n].value = -1;

	is_str++;
	n++;

	props[n].name = MakeAtom("QUAD_WIDTH", sizeof("QUAD_WIDTH") - 1, TRUE);
	props[n].value = vals->width;

	is_str++;
	n++;

	/* nprops */
	ptr->nprops = n;

	dprintf((stdout, "n %d\n", n));
	dprintf((stdout, "-" __FUNCTION__ "()\n\n" ));
	return 0;
}

/***************************************************************************/

/*
 * initialize Type1_16Font struct
 */

static int Type1_16Font_Init(FontPtr font,
			     char* dir,
			     char* file,
			     double matrix[4])
{
	char fontName[MAXFONTFILENAMELEN];
	int pos;
	Type1_16FontPtr ptr;
	int wbytes;

	dprintf((stdout, "\n+Type1_16Font_Init(0x%x)\n", font));

	strcpy(fontName, file);
	pos = strlen(fontName) - strlen(T1_16_EXT);
	if (pos <= 0) {
		return BadFontName;
	}
	fontName[pos] = '\0';

	dprintf((stdout, "fontname %s\n", fontName));

	ptr = (Type1_16FontPtr) xalloc(sizeof(Type1_16Font));
	if (!ptr) {
		return AllocError;
	}

	ptr->fontname = (char *)xalloc(strlen(fontName) +1 );
	if(ptr->fontname == NULL) {
		xfree(ptr);
		return AllocError;
	}
	strcpy(ptr->fontname, fontName);



	ptr->font = font;

	wbytes = (d_to_i(computeWidth(matrix)) + 7) / 8;
	wbytes += font->glyph - 1;
	wbytes -= wbytes % font->glyph;
	ptr->wbytes = wbytes;

	ptr->hbits = d_to_i(computeHeight(matrix));

	ptr->matrix[0] = matrix[0];
	ptr->matrix[1] = matrix[1];
	ptr->matrix[2] = matrix[2];
	ptr->matrix[3] = matrix[3];


	ptr->pool = CharInfoPool_Alloc(ptr->wbytes * ptr->hbits);
	if (ptr->pool == NULL) {
		xfree(ptr);
		return AllocError;
	}

	ptr->btree = BTree_Alloc();
	if (ptr->btree == NULL) {
		CharInfoPool_Free(ptr->pool);
		xfree(ptr);
		return AllocError;
	}

	if (font->bit == MSBFirst) {
		if (font->byte == LSBFirst) {
			ptr->convert = convertNothing;
		} else {
			ptr->convert = convertByteOrder;
		}
	} else {
		if (font->byte == LSBFirst) {
			ptr->convert = convertBitOrder;
		} else {
			ptr->convert = convertBitByteOrder;
		}
	}

	font->fontPrivate = (pointer) ptr;

	dprintf((stdout, "-Type1_16Font_Init()\n\n"));
	return Successful;
}


/* եȤɬפʤʤä */
static void Type1_16Font_Deinit(FontPtr font)
{
	Type1_16FontPtr ptr = (Type1_16FontPtr) font->fontPrivate;

	dprintf((stdout, "\n+Type1_16Font_Deinit(0x%x)\n", font));


	/* β */
	CharInfoPool_Free(ptr->pool);
	BTree_Free(ptr->btree);
	xfree(ptr->fontname);
	xfree(ptr);
	xfree(font->info.props);
	xfree(font->info.isStringProp);
	xfree(font);

	dprintf((stdout, "-Type1_16Font_Deinit()\n\n"));
}

/*************************************************************************/

/* 饯¸ߤϰϤꤹ */
static int Type1_16SetupRowCol(FontPtr         font,
			       FontEntryPtr    entry,
			       FontScalablePtr vals)
{

	if (strcasestr(entry->name.name, "jisx0208") ||        /* JISX0208 */
	    strcasestr(vals->xlfdName, "jisx0208")) {
		dprintf((stdout, "jisx0208\n"));
		font->info.firstCol = 0x21;
		font->info.lastCol  = 0x7e;
		font->info.firstRow = 0x21;
		font->info.lastRow  = 0x74;
		font->info.defaultCh = 0x2121;
	} else if (strcasestr(entry->name.name, "jisx0212") || /* JISX0212 */
		   strcasestr(vals->xlfdName, "jisx0212")) {
		dprintf((stdout, "jisx0212\n"));
		font->info.firstCol = 0x21;
		font->info.lastCol  = 0x7e;
		font->info.firstRow = 0x22;
		font->info.lastRow  = 0x6d;
		font->info.defaultCh = 0x2221;
	} else if (strcasestr(entry->name.name, "big5") ||      /* BIG5 */
		   strcasestr(vals->xlfdName, "big5")) {
		font->info.firstCol = 0x40;
		font->info.lastCol  = 0xfe;
		font->info.firstRow = 0xa1;
		font->info.lastRow  = 0xfa;
		font->info.defaultCh = 0xa140;
	} else if (strcasestr(entry->name.name, "gb2312.1980") ||/* GB2312 */
		   strcasestr(vals->xlfdName, "gb2312.1980")) {
		font->info.firstCol = 0x21;
		font->info.lastCol  = 0x7e;
		font->info.firstRow = 0x21;
		font->info.lastRow  = 0x77;
		font->info.defaultCh = 0x2121;
	} else if (strcasestr(entry->name.name, "gb2312.80&gb8565.88") ||/* GB2312&gb8565 */
		   strcasestr(vals->xlfdName, "gb2312.80&gb8565.88")) {
		font->info.firstCol = 0x21;
		font->info.lastCol  = 0x7e;
		font->info.firstRow = 0x21;
		font->info.lastRow  = 0x7e;
		font->info.defaultCh = 0x2121;
	} else if (strcasestr(entry->name.name, "jisc6226") || /* JISC6226 */
		   strcasestr(vals->xlfdName, "jisc6226")) {
		font->info.firstCol = 0x21;
		font->info.lastCol  = 0x7e;
		font->info.firstRow = 0x21;
		font->info.lastRow  = 0x77;
		font->info.defaultCh = 0x2121;
	} else if (strcasestr(entry->name.name, "ksc5601") || /* KSC5601 */
		   strcasestr(vals->xlfdName, "ksc5601")) {
		font->info.firstCol = 0x21;
		font->info.lastCol  = 0x7e;
		font->info.firstRow = 0x21;
		font->info.lastRow  = 0x7d;
		font->info.defaultCh = 0x2121;
#if 0
	} else if (strcasestr(entry->name.name, "iso8859") || /* ISO8859 */
		   strcasestr(vals->xlfdName, "iso8859")) {
		font->info.firstCol = 0x20;
		font->info.lastCol  = 0xff;
		font->info.firstRow = 0x00;
		font->info.lastRow  = 0x00;
		font->info.defaultCh = 0x0020;
#endif
	} else {                                                /* ¾ */
		return BadFontName;
	}

	return Successful;
}

/* scaleable font loader entry point */
int Type1_16OpenScalable(FontPathElementPtr fpe,
			 FontPtr	    *pFont,
			 int                flags,
			 FontEntryPtr       entry,
			 char               *fileName,
			 FontScalablePtr    vals,
			 fsBitmapFormat     format,
			 fsBitmapFormatMask fmask,
			 FontPtr            non_cachable_font) /* We don't do licensing */
{
	char xlfdName[MAXFONTNAMELEN];
	char fontName[MAXFONTFILENAMELEN];
	FontPtr font;
	int w, h, ch_w, ch_h, ascent, descent;
	int ret;
	int image;

	dprintf((stdout, "\n+Type1_16OpenScalable(%x, %x, %x, %x, %s, %x, %x, %x, %x)\n",
	 fpe, pFont, flags, entry, fileName, vals, format, fmask, non_cachable_font));
#ifdef DUMP
/*	DumpFontScalable(vals);
	DumpFontEntry(entry);*/
#endif

	strncpy(xlfdName, vals->xlfdName, sizeof(xlfdName));
	FontParseXLFDName(xlfdName, vals, FONT_XLFD_REPLACE_VALUE);

	dprintf((stdout, "\n" __FUNCTION__ ":%s -> %s\n\n", vals->xlfdName, xlfdName));
#if 0
	/* Reject outrageously small font sizes to keep the math from
	   blowing up. */
	if ((hypot(vals->pixel_matrix[0], vals->pixel_matrix[2]) < 1.0) ||
	    (hypot(vals->pixel_matrix[1], vals->pixel_matrix[3]) < 1.0)) {
		return BadFontName;
	}
#endif /* 0 */

	switch (entry->type) {
	case FONT_ENTRY_SCALABLE:
		strncpy(fontName, entry->u.scalable.fileName, sizeof(fontName));
		break;
	case FONT_ENTRY_SCALE_ALIAS:
		return BadFontName;
	case FONT_ENTRY_BITMAP:
		strncpy(fontName, entry->u.bitmap.fileName, sizeof(fontName));
		break;
	case FONT_ENTRY_ALIAS:
		return BadFontName;
	case FONT_ENTRY_BC:
		return BadFontName;
	default:
		return BadFontName;
	}

	font = (FontPtr) xalloc(sizeof(FontRec));
	if (!font) {
		return AllocError;
	}

	/* 饯礭 */
	w = d_to_i(computeWidth(vals->pixel_matrix));
	h = d_to_i(computeHeight(vals->pixel_matrix));
	ch_w = d_to_i(computeChWidth(vals->pixel_matrix));
	ch_h = d_to_i(computeChHeight(vals->pixel_matrix));

	ascent = (h * 4) / 5;
	descent = h - ascent;

	vals->pixel = ch_h;
	vals->width = ch_w;

	/* 饯¸ߤϰϤꤹ */
	ret = Type1_16SetupRowCol(font, entry, vals);
	if (ret != Successful) {
		xfree(font);
		return ret;
	}

	font->refcnt = 0;
	font->info.noOverlap = 1;
	font->info.terminalFont = 1;
	font->info.constantMetrics = 1;
	font->info.constantWidth = 1;
	font->info.inkInside = 1;
	font->info.inkMetrics = 1;
	font->info.allExist = 0;
	font->info.drawDirection = LeftToRight;
	font->info.cachable = 1;
	font->info.anamorphic = 0;
	font->info.maxOverlap = 0;
	font->info.pad = (short)0xf0f0; /* 0, 0xf0f0 ??? */
	font->info.maxbounds.leftSideBearing = 0;
	font->info.maxbounds.rightSideBearing = w;
	font->info.maxbounds.characterWidth = ch_w;
	font->info.maxbounds.ascent = ascent;
	font->info.maxbounds.descent = descent;
	font->info.maxbounds.attributes = 0x00;
	font->info.minbounds.leftSideBearing = 0;
	font->info.minbounds.rightSideBearing = w;
	font->info.minbounds.characterWidth = ch_w;
	font->info.minbounds.ascent = ascent;
	font->info.minbounds.descent = descent;
	font->info.minbounds.attributes = 0x00;
	/* 10/14 : 0.7142 < 0.7857
	   11/16 : 0.6875 < 0.75
	   17/24 : 0.7083 < 0.75 */
	font->info.ink_maxbounds.leftSideBearing = (int)(w * 0.725);
	font->info.ink_maxbounds.rightSideBearing = w;
	font->info.ink_maxbounds.characterWidth = ch_w;
	font->info.ink_maxbounds.ascent = ascent;
	font->info.ink_maxbounds.descent = descent;
	font->info.ink_maxbounds.attributes = 0x00;
	font->info.ink_minbounds.leftSideBearing = 0;
	font->info.ink_minbounds.rightSideBearing = 0;
	font->info.ink_minbounds.characterWidth = ch_w;
	/* -1/14, -1/16,  -1/24 */
	font->info.ink_minbounds.ascent = -1; /* ??? */
	/* -11/14, -13/16  -21/24 */
	font->info.ink_minbounds.descent = -(ascent - 1); /* ??? */
	font->info.ink_minbounds.attributes = 0x00;
	font->info.fontAscent = ascent;
	font->info.fontDescent = descent;
	if (Type1_16MakeProps(&font->info, vals) < 0) {
		font->info.nprops = 0;
		font->info.props = NULL;
		font->info.isStringProp = NULL;
	}

        /*
	  font->bit = MSBFirst;
	  font->byte = LSBFirst;
	  font->glyph = 4;
	  font->scan = 1;
	  */
	font->scan = 1;

	FontDefaultFormat(&(font->bit), &(font->byte),
			  &(font->glyph), &(font->scan));
	ret = CheckFSFormat(format, fmask,
			    &(font->bit), &(font->byte),
			    &(font->scan), &(font->glyph),
			    &image);
	if (ret != Successful) {
		xfree(font);
		return ret;
	}
	font->format = format;
	font->get_glyphs = t1_16_get_glyphs;
	font->get_metrics = t1_16_get_metrics;
	font->unload_font = t1_16_unload_font;
	font->unload_glyphs = t1_16_unload_glyphs;
        /*
	  font->fpe = ;
	  font->svrPrivate = ;
	  */
        /*
	  font->fontPrivate = ;
	  */


	ret = Type1_16Font_Init(font, fpe->name, entry->u.scalable.fileName, vals->pixel_matrix);
	if (ret != Successful) {
		xfree(font);
		return ret;
	}
        /*
	  font->fpePrivate = ;
	  */
	font->maxPrivate = -1;
	font->devPrivates = NULL;


#ifdef DUMP
	/*DumpFont(font);*/
#endif

	*pFont = font;

	dprintf((stdout, "FontPtr 0x%x\n", *pFont));

	dprintf((stdout, "-Type1_16OpenScalable()\n\n"));
	return Successful;

}

int Type1_16GetInfoScalable(FontPathElementPtr  fpe,
			    FontInfoPtr	        pFontInfo,
			    FontEntryPtr        entry,
			    FontNamePtr	        pFontName,
			    char	        *fileName,
			    FontScalablePtr	vals)
{
	FontPtr pfont;
	int flags = 0;
	long format = 0;  /* It doesn't matter what format for just info */
	long fmask = 0;
	int ret;

	dprintf((stderr, "\nType1_16GetInfoScalable(%x, %x, %x, %x, %s, %x)\n",
		 fpe, pFontInfo, entry, pFontName, fileName, vals));

	ret = Type1_16OpenScalable(fpe, &pfont, flags, entry, fileName, vals,
				   format, fmask, NULL);
	if (ret != Successful)
		return ret;
	*pFontInfo = pfont->info;

	(*pfont->unload_font)(pfont);

	return Successful;
}


static FontRendererRec renderer[] = {
	{T1_16_EXT, T1_16_EXT_LEN,
	 (int (*)()) 0, Type1_16OpenScalable,
	 (int (*)()) 0, Type1_16GetInfoScalable,
	 0, CAP_MATRIX | CAP_CHARSUBSETTING,},
	{T1_16_EXT_I, T1_16_EXT_LEN_I,
	 (int (*)()) 0, Type1_16OpenScalable_italic,
	 (int (*)()) 0, Type1_16GetInfoScalable_italic,
	 0, CAP_MATRIX | CAP_CHARSUBSETTING,},
};

void Type1_16RegisterFontFileFunctions(void)
{
	int i;

	dprintf((stdout, "\n+Type1_16RegisterFontFileFunctions()\n"));

	signal(SIGCHLD, CleanupChild);
	signal(SIGALRM, gsDeinit);
	atexit(gsAtExit);

	for (i = 0; i < XtNumber(renderer); i++) {
		FontFileRegisterRenderer(&(renderer[i]));
	}
	dprintf((stdout, "-Type1_16RegisterFontFileFunctions()\n\n"));
}

/* end */
