/*******************************************************************
 * Xatm -- OCF, CID, TrueType scalable font support for X 
 *  By:  ͵ <y5t0964@students.chiba-u.ac.jp>
 */

/* This code is derived from X-VFlib's vffuncs.c */

/* dump.c */
/* ǥХå */

/*$Id: dump.c,v 1.5 1998/05/31 15:06:49 elca Exp $*/

#include "Type1_16.h"

static void DumpFontPathElement(FontPathElementPtr ptr);
static void DumpxCharInfo(xCharInfo *ptr);
static void DumpFontInfo(FontInfoPtr ptr);
static void DumpFont(FontPtr ptr);
static void DumpFontName(FontNamePtr ptr);
static void DumpFontEntry(FontEntryPtr ptr);
static void DumpfsRange(fsRange *ptr);
static void DumpFontScalable(FontScalablePtr ptr);

static void DumpFontPathElement(FontPathElementPtr ptr)
{
  fprintf(stdout, "FontPathElement 0x%x\n", ptr);
  if ((ptr == NULL) || ((unsigned) ptr >= 0xf0000000)) {
    return;
  }
  fprintf(stdout, "  name_length %d\n", ptr->name_length);
  fprintf(stdout, "  name \"%s\"\n", ptr->name);
  fprintf(stdout, "  type %d\n", ptr->type);
  fprintf(stdout, "  refcount %d\n", ptr->refcount);
  fprintf(stdout, "  private 0x%x\n", ptr->private);
  fprintf(stdout, "\n");
}

static void DumpxCharInfo(xCharInfo *ptr)
{
  fprintf(stdout, "FontInfo 0x%x\n", ptr);
  if (ptr == NULL) {
    return;
  }
  fprintf(stdout, "  leftSideBearing %d\n", ptr->leftSideBearing);
  fprintf(stdout, "  rightSideBearing %d\n", ptr->rightSideBearing);
  fprintf(stdout, "  characterWidth %d\n", ptr->characterWidth);
  fprintf(stdout, "  ascent %d\n", ptr->ascent);
  fprintf(stdout, "  descent %d\n", ptr->descent);
  fprintf(stdout, "  attributes 0x%x\n", ptr->attributes);
  fprintf(stdout, "\n");
}

static void DumpFontInfo(FontInfoPtr ptr)
{
  fprintf(stdout, "FontInfo 0x%x\n", ptr);
  if (ptr == NULL) {
    return;
  }
  fprintf(stdout, "  firstCol 0x%x\n", ptr->firstCol);
  fprintf(stdout, "  lastCol 0x%x\n", ptr->lastCol);
  fprintf(stdout, "  firstRow 0x%x\n", ptr->firstRow);
  fprintf(stdout, "  lastRow 0x%x\n", ptr->lastRow);
  fprintf(stdout, "  defaultCh 0x%x\n", ptr->defaultCh);
  fprintf(stdout, "  noOverlap %d\n", ptr->noOverlap);
  fprintf(stdout, "  terminalFont %d\n", ptr->terminalFont);
  fprintf(stdout, "  constantMetrics %d\n", ptr->constantMetrics);
  fprintf(stdout, "  constantWidth %d\n", ptr->constantWidth);
  fprintf(stdout, "  inkInside %d\n", ptr->inkInside);
  fprintf(stdout, "  inkMetrics %d\n", ptr->inkMetrics);
  fprintf(stdout, "  allExist %d\n", ptr->allExist);
  fprintf(stdout, "  drawDirection %d\n", ptr->drawDirection);
  fprintf(stdout, "  cachable %d\n", ptr->cachable);
  fprintf(stdout, "  anamorphic %d\n", ptr->anamorphic);
  fprintf(stdout, "  maxOverlap %d\n", ptr->maxOverlap);
  fprintf(stdout, "  pad 0x%x\n", ptr->pad);
  fprintf(stdout, "  maxbounds ->\n"); DumpxCharInfo(&ptr->maxbounds);
  fprintf(stdout, "  minbounds ->\n"); DumpxCharInfo(&ptr->minbounds);
  fprintf(stdout, "  ink_maxbounds ->\n"); DumpxCharInfo(&ptr->ink_maxbounds);
  fprintf(stdout, "  ink_minbounds ->\n"); DumpxCharInfo(&ptr->ink_minbounds);
  fprintf(stdout, "  fontAscent %d\n", ptr->fontAscent);
  fprintf(stdout, "  fontDescent %d\n", ptr->fontDescent);
  fprintf(stdout, "  nprops %d\n", ptr->nprops);
  fprintf(stdout, "  props -> 0x%x\n", ptr->props);
  fprintf(stdout, "  isStringProp \"%s\"\n", ptr->isStringProp);
  fprintf(stdout, "\n");
}

static void DumpFont(FontPtr ptr)
{
  fprintf(stdout, "Font 0x%x\n", ptr);
  if (ptr == NULL) {
    return;
  }
  fprintf(stdout, "  refcnt %d\n", ptr->refcnt);
  fprintf(stdout, "  info ->\n"); DumpFontInfo(&ptr->info);
  fprintf(stdout, "  bit 0x%x\n", ptr->bit);
  fprintf(stdout, "  byte 0x%x\n", ptr->byte);
  fprintf(stdout, "  glyph 0x%x\n", ptr->glyph);
  fprintf(stdout, "  scan 0x%x\n", ptr->scan);
  fprintf(stdout, "  format %x\n", ptr->format);
  fprintf(stdout, "  get_glyphs 0x%x\n", ptr->get_glyphs);
  fprintf(stdout, "  get_metrics 0x%x\n", ptr->get_metrics);
  fprintf(stdout, "  unload_font 0x%x\n", ptr->unload_font);
  fprintf(stdout, "  unload_glyphs 0x%x\n", ptr->unload_glyphs);
  fprintf(stdout, "  fpe ->\n"); DumpFontPathElement(ptr->fpe);
  fprintf(stdout, "  svrPrivate 0x%x\n", ptr->svrPrivate);
  fprintf(stdout, "  fontPrivate 0x%x\n", ptr->fontPrivate);
  fprintf(stdout, "  fpePrivate 0x%x\n", ptr->fpePrivate);
  fprintf(stdout, "  maxPrivate %d\n", ptr->maxPrivate);
  fprintf(stdout, "  devPrivates 0x%x\n", ptr->devPrivates);
  fprintf(stdout, "\n");
}

static void DumpFontName(FontNamePtr ptr)
{
  fprintf(stdout, "FontName 0x%x\n", ptr);
  if (ptr == NULL) {
    return;
  }
  fprintf(stdout, "  name \"%s\"\n", ptr->name);
  fprintf(stdout, "  length %d\n", ptr->length);
  fprintf(stdout, "  ndashes %d\n", ptr->ndashes);
  fprintf(stdout, "\n");
}

static void DumpFontEntry(FontEntryPtr ptr)
{
  fprintf(stdout, "FontEntry 0x%x\n", ptr);
  if (ptr == NULL) {
    return;
  }
  fprintf(stdout, "  name ->\n"); DumpFontName(&ptr->name);
  fprintf(stdout, "  type %d\n", ptr->type);
  switch (ptr->type) {
  case FONT_ENTRY_SCALABLE:
    fprintf(stdout, "  u.scalable.renderer -> 0x%x\n", ptr->u.scalable.renderer);
    fprintf(stdout, "  u.scalable.fileName \"%s\"\n", ptr->u.scalable.fileName);
    fprintf(stdout, "  u.scalable.extra -> 0x%x\n", ptr->u.scalable.extra);
    break;
  case FONT_ENTRY_SCALE_ALIAS:
    break;
  case FONT_ENTRY_BITMAP:
    fprintf(stdout, "  u.bitmap.renderer -> 0x%x\n", ptr->u.bitmap.renderer);
    fprintf(stdout, "  u.bitmap.fileName \"%s\"\n", ptr->u.bitmap.fileName);
    fprintf(stdout, "  u.bitmap.pFont -> 0x%x\n", ptr->u.bitmap.pFont);
    break;
  case FONT_ENTRY_ALIAS:
    fprintf(stdout, "  u.alias.resolved \"%s\"\n", ptr->u.alias.resolved);
    break;
  case FONT_ENTRY_BC:
    fprintf(stdout, "  u.bc.vals ???\n");
    fprintf(stdout, "  u.bc.entry ->\n"); DumpFontEntry(ptr->u.bc.entry);
    break;
  }
  fprintf(stdout, "\n");
}

static void DumpfsRange(fsRange *ptr)
{
  fprintf(stdout, "fsRange 0x%x\n", ptr);
  if (ptr == NULL) {
    return;
  }
  fprintf(stdout, "  min_char_high %d\n", ptr->min_char_high);
  fprintf(stdout, "  min_char_low %d\n", ptr->min_char_low);
  fprintf(stdout, "  max_char_high %d\n", ptr->max_char_high);
  fprintf(stdout, "  max_char_low %d\n", ptr->max_char_low);
  fprintf(stdout, "\n");
}

static void DumpFontScalable(FontScalablePtr ptr)
{
  int i;
  fprintf(stdout, "FontScalable 0x%x\n", ptr);
  if (ptr == NULL) {
    return;
  }
  fprintf(stdout, "  values_supplied 0x%x\n", ptr->values_supplied);
  fprintf(stdout, "  pixel_matrix[] = {%g, %g, %g, %g}\n",
	  ptr->pixel_matrix[0], ptr->pixel_matrix[1],
	  ptr->pixel_matrix[2], ptr->pixel_matrix[3]);
  fprintf(stdout, "  point_matrix[] = {%g, %g, %g, %g}\n",
	  ptr->point_matrix[0], ptr->point_matrix[1],
	  ptr->point_matrix[2], ptr->point_matrix[3]);
  fprintf(stdout, "  pixel %d\n", ptr->pixel);
  fprintf(stdout, "  point %d\n", ptr->point);
  fprintf(stdout, "  x %d\n", ptr->x);
  fprintf(stdout, "  y %d\n", ptr->y);
  fprintf(stdout, "  width %d\n", ptr->width);
  fprintf(stdout, "  xlfdName \"%s\"\n", ptr->xlfdName);
  fprintf(stdout, "  nranges %d\n", ptr->nranges);
/*  for (i = 0; i < ptr->nranges; i++) {
    DumpfsRange(&ptr->ranges[i]);
  }*/
  fprintf(stdout, "\n");
}

/* end */
